//////////////////////////////////////////////////////////////////////
//       SpeedStep Centrino © Gergely Rózsahegyi (RoGer) 2005       //
//                           roger@beos.hu                          //
//                                                                  //
//                      SpeedStep  Source File                      //
//                                                                  //
//                       Created:  2005.05.23                       //
//                                                                  //
//////////////////////////////////////////////////////////////////////

#include <unistd.h>
#include <fcntl.h>
#include <iostream>
#include <Message.h>
#include "Driver/SpeedStep.h"
#include <String.h>

using namespace std;

void print(unsigned buf, char *str)
{
	unsigned freq, volt;
	freq = buf;
	freq = (freq >> 8) & 0xff;
	freq *= 100;
	cout << str<< " frequency:\t" << freq << "MHz" << endl;

	volt = buf;
	volt &=  0xff;
	cout << str << " voltage:\t" << (float)(volt * 16 + 700) / 1000 << "V" << endl << endl;

}

int main(int argc, char *argv[])
{
	uint p_count = 0;
	while (current_table[p_count].freq != 0)
	{
		p_count++;
	}

	if (argc > 1)
	{
		if(strcmp(argv[1], "-help") == 0 || strcmp(argv[1], "-h") == 0)
		{
			cout << endl;
			cout << "\tspeedstep\tShow current status." << endl;
			int i = 0;
			while (current_table[i].freq != 0)
			{
				cout << "\tspeedstep -P"<< i << "\tSet P" << i << " frequency and voltage. (" << (current_table[i].freq < 1000 ? " " : "") << current_table[i].freq << "MHz, " << (float)current_table[i].volt / 1000 << "V)." << endl;
				i++;
			}
			cout << "\tspeedstep -help\tShow this help." << endl;
			cout << "\tspeedstep -gui\tShow current status for GUI program." << endl;
			return 0;
		}
	}

	int fd = open("/dev/misc/speedstep_centrino", O_RDWR | O_NONBLOCK);
	
	if(!fd) {
		cout << "error: please check if the driver is installed\n";
		return(true);
	}
	
	unsigned buf;
	size_t len = 100;
	read(fd, &buf, len);

	if (argc <= 1)
	{
		cout << endl;
		print(buf, "\tCurrent");
		return(0);
	}

	status_t ret;
	if(strcmp(argv[1], "-P0") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P0, NULL, 0);
	else
	if(strcmp(argv[1], "-P1") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P1, NULL, 0);
	else
	if(strcmp(argv[1], "-P2") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P2, NULL, 0);
	else
	if(strcmp(argv[1], "-P3") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P3, NULL, 0);
	else
	if(strcmp(argv[1], "-P4") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P4, NULL, 0);
	else
	if(strcmp(argv[1], "-P5") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P5, NULL, 0);
	else
	if(strcmp(argv[1], "-P6") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P6, NULL, 0);
	else
	if(strcmp(argv[1], "-P7") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P7, NULL, 0);
	else
	if(strcmp(argv[1], "-P8") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P8, NULL, 0);
	else
	if(strcmp(argv[1], "-P9") == 0 && p_count > 0)
		ret = ioctl(fd, SPEEDSTEP_SET_P9, NULL, 0);
	else
	if(strcmp(argv[1], "-gui") == 0)
	{
		unsigned freq, volt;
		freq = buf;
		freq = (freq >> 8) & 0xff;
		freq *= 100;
		volt = buf;
		volt &=  0xff;
		volt = volt * 16 + 700;
		cout << freq << endl << volt << endl;
		return 0;
	}
	else
	{
		cout << endl;
		print(buf, "\tCurrent");
		return 0;
	}

	cout << endl;
	print(buf, "\tOld");

	if (ret == 0)
	{
		buf = 0;
		snooze(100000);
		read(fd, &buf, len);
		print(buf, "\tNew");
	} else
		cout << "Couldn't change frequency." << endl;
	
	return 0;	
}
